//Blink LED in the Start Task

#include  <bsp_os.h>
#include  "bsp.h"

#include  <cpu/include/cpu.h>
#include  <common/include/common.h>
#include  <kernel/include/os.h>
#include  <kernel/include/os_trace.h>

#include  <common/include/lib_def.h>
#include  <common/include/rtos_utils.h>
#include  <common/include/toolchains.h>
#include  "retargetserial.h"

/********************************************************************************************************
*                                             LOCAL DEFINES
********************************************************************************************************/
#define  START_TASK_PRIO              21u								// Start Task priority
#define  START_TASK_STK_SIZE         256u								// Start Task stack size
#define  MY_TASK_STK_SIZE			 256u

/********************************************************************************************************
*                                        LOCAL GLOBAL VARIABLES
********************************************************************************************************/
static  CPU_STK StartTaskStk[START_TASK_STK_SIZE];  					// Start Task Stack.
static  CPU_STK	ButtonTaskStk[MY_TASK_STK_SIZE];
static  CPU_STK	LampTaskStk[MY_TASK_STK_SIZE];

static  OS_TCB  StartTaskTCB;											// Start Task TCB.
static  OS_TCB	ButtonTaskTCB;
static  OS_TCB	LampTaskTCB;

static 	OS_Q      	Button_q;

/*********************************************************************************************************
*                                       LOCAL FUNCTION PROTOTYPES
*********************************************************************************************************/
static void StartTask(void *p_arg);										// Start Task prototype
static void	ButtonTask (void *p);
static void	LampTask (void *p);

int main (void)
{
    RTOS_ERR  err;

    BSP_SystemInit();                                           		// Initialize System.
    CPU_Init();                                                 		// Initialize CPU.

    RETARGET_SerialInit();
    RETARGET_SerialCrLf(1);

    OS_TRACE_INIT();
    OSInit(&err);                                               		// Initialize the Kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSTaskCreate(&StartTaskTCB,                          				// Create the Start Task.
                 "Start Task",
                  StartTask,
                  DEF_NULL,
                  START_TASK_PRIO,
                 &StartTaskStk[0],
                 (START_TASK_STK_SIZE / 10u),
                  START_TASK_STK_SIZE,
                  0u,
                  0u,
                  DEF_NULL,
                 (OS_OPT_TASK_STK_CLR),
                 &err);
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    OSStart(&err);                                              		// Start the kernel.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), 1);	//   Check error code.

    return (1);
}

static void StartTask (void *p_arg)
{
    RTOS_ERR  err;
    PP_UNUSED_PARAM(p_arg);                                     		// Prevent compiler warning.
    BSP_TickInit();                                             		// Initialize Kernel tick source.

#if (OS_CFG_STAT_TASK_EN == DEF_ENABLED)
    OSStatTaskCPUUsageInit(&err);                               		// Initialize CPU Usage.
    APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;);	//   Check error code.
#endif

#ifdef CPU_CFG_INT_DIS_MEAS_EN
    CPU_IntDisMeasMaxCurReset();                                		// Initialize interrupts disabled measurement.
#endif

    Common_Init(&err);                                          		// Call common module initialization example.
    APP_RTOS_ASSERT_CRITICAL(err.Code == RTOS_ERR_NONE, ;);

    BSP_OS_Init();                                              		// Initialize the BSP. It is expected that the BSP ...
                                                                		// ... will register all the hardware controller to ...
                                                                		// ... the platform manager at this moment.
    GPIO_PinModeSet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN, gpioModeInput, 0);
    GPIO_PinModeSet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB1_PIN, gpioModeInput, 0);

    OSTaskCreate(&ButtonTaskTCB,           								// Create first custom tasks
                     "Button Task",
                     ButtonTask,
                     0u,
    				 START_TASK_PRIO+1,
                     &ButtonTaskStk[0u],
                     ButtonTaskStk[MY_TASK_STK_SIZE / 10u],
                     MY_TASK_STK_SIZE,
                     0u,
                     0u,
                     0u,
                     (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                     &err);

        OSTaskCreate(&LampTaskTCB,              					// Create second custom task
                     "Lamp Task",
                      LampTask,
                      0u,
                      START_TASK_PRIO+2,
                      &LampTaskStk[0u],
                      LampTaskStk[MY_TASK_STK_SIZE / 10u],
                      MY_TASK_STK_SIZE,
                      0u,
                      0u,
                      0u,
                      (OS_OPT_TASK_STK_CHK | OS_OPT_TASK_STK_CLR),
                      &err);

    while (1)
    {
//        BSP_LedToggle(0);
        OSTimeDlyHMSM(1u, 0u, 0u, 100u, OS_OPT_TIME_HMSM_STRICT, &err);
        APP_RTOS_ASSERT_DBG((RTOS_ERR_CODE_GET(err) == RTOS_ERR_NONE), ;); //   Check error code.
    }
}

static void ButtonTask (void *p_arg)
{
    RTOS_ERR err;
    PP_UNUSED_PARAM(p_arg);                                     // Prevent compiler warning
    uint32_t button0_pressed, button0_state = 1,				// 1 = off, 0 = pressed
    		 button1_pressed, button1_state = 1;
    uint8_t  LEDno;

    OSQCreate (&Button_q, "My queue", 20, &err);				// init queue for 20 bytes


    while (DEF_TRUE)
    {
    	button0_pressed = GPIO_PinInGet(BSP_GPIO_PB0_PORT, BSP_GPIO_PB0_PIN); // current button state, 0 = pressed
    	if (button0_state == 1)          						// old state = "OFF"
    	{
    		if (button0_pressed == 0)
    		{													// button press is detected
    			button0_state = 0;             					// new status
    			LEDno = 0;
    			OSQPost(&Button_q, &LEDno, sizeof(LEDno), OS_OPT_POST_FIFO, &err);  		// unlock the LampTask
    	    }
    	}
    	else                           							// old status is "ON"
    	{
    		if (button0_pressed != 0)
    		{                            						// button release is detected
    	       button0_state = 1;           					// just update button state
    	    }
    	}

    	button1_pressed = GPIO_PinInGet(BSP_GPIO_PB1_PORT, BSP_GPIO_PB1_PIN); // current button state, 0 = pressed
    	if (button1_state == 1)          						// old state = "OFF"
    	{
    		if (button1_pressed == 0)
    		{													// button press is detected
    			button1_state = 0;             					// new status
    			LEDno = 1;
    			OSQPost(&Button_q, &LEDno, sizeof(LEDno), OS_OPT_POST_ALL, &err);  		// unlock the LampTask
    	    }
    	}
    	else                           							// old status is "ON"
    	{
    		if (button1_pressed != 0)
    		{                            						// button release is detected
    	       button1_state = 1;           					// just update button state
    	    }
    	}

    	OSTimeDlyHMSM(0u, 0u, 0u, 100u, OS_OPT_TIME_HMSM_STRICT, &err);
    }
}

static void LampTask (void *p_arg)
{
    RTOS_ERR err;
    PP_UNUSED_PARAM(p_arg);                                     // Prevent compiler warning
    uint8_t ledNO;
    OS_MSG_SIZE msgSize;

    while (DEF_TRUE)
    {
    	ledNO = *(uint8_t*) OSQPend(&Button_q, 0, OS_OPT_PEND_BLOCKING, &msgSize, 0, &err);	// wait for semaphore
    	BSP_LedClear(0);										// turn off both LEDs
    	BSP_LedClear(1);
    	BSP_LedSet(ledNO);										// turn on one of them
    }
}

